@@ -1,6 +1,5 @@
 #ifndef ERRLOG_H_INCLUDED
 #define ERRLOG_H_INCLUDED
-#define ERRLOG_H_VERSION "$Id: errlog.h,v 1.30 2013/11/24 14:23:28 fabiankeil Exp $"
 /*********************************************************************
  *
  * File        :  $Source: /cvsroot/ijbswa/current/errlog.h,v $
@@ -8,8 +7,8 @@
  * Purpose     :  Log errors to a designated destination in an elegant,
  *                printf-like fashion.
  *
- * Copyright   :  Written by and Copyright (C) 2001-2009 the SourceForge
- *                Privoxy team. http://www.privoxy.org/
+ * Copyright   :  Written by and Copyright (C) 2001-2009 members of the
+ *                Privoxy team. https://www.privoxy.org/
  *
  *                Based on the Internet Junkbuster originally written
  *                by and Copyright (C) 1997 Anonymous Coders and
@@ -38,8 +37,7 @@
 
 /* Debug level for errors */
 
-/* XXX: Should be renamed. */
-#define LOG_LEVEL_GPC        0x0001
+#define LOG_LEVEL_REQUEST    0x0001
 #define LOG_LEVEL_CONNECT    0x0002
 #define LOG_LEVEL_IO         0x0004
 #define LOG_LEVEL_HEADER     0x0008
@@ -55,6 +53,14 @@
 #define LOG_LEVEL_CGI        0x0800 /* CGI / templates */
 #define LOG_LEVEL_RECEIVED   0x8000
 #define LOG_LEVEL_ACTIONS   0x10000
+#ifdef FUZZ
+/*
+ * Permanently disables logging through log_error().
+ * Useful to reduce pointless overhead when fuzzing
+ * without watching stdout.
+ */
+#define LOG_LEVEL_STFU      0x20000
+#endif
 
 /* Following are always on: */
 #define LOG_LEVEL_INFO    0x1000
@@ -68,11 +74,7 @@ extern void disable_logging(void);
 extern void init_log_module(void);
 extern void show_version(const char *prog_name);
 extern void log_error(int loglevel, const char *fmt, ...);
-extern const char *jb_err_to_string(int jb_error);
-
-/* Revision control strings from this header and associated .c file */
-extern const char errlog_rcs[];
-extern const char errlog_h_rcs[];
+extern const char *jb_err_to_string(jb_err jb_error);
 
 #endif /* ndef ERRLOG_H_INCLUDED */
 
