@@ -1,4 +1,3 @@
-const char loaders_rcs[] = "$Id: loaders.c,v 1.74 2009/09/26 13:29:57 fabiankeil Exp $";
 /*********************************************************************
  *
  * File        :  $Source: /cvsroot/ijbswa/current/loaders.c,v $
@@ -8,8 +7,8 @@ const char loaders_rcs[] = "$Id: loaders.c,v 1.74 2009/09/26 13:29:57 fabiankeil
  *                the list of active loaders, and to automatically
  *                unload files that are no longer in use.
  *
- * Copyright   :  Written by and Copyright (C) 2001-2009 the
- *                Privoxy team. http://www.privoxy.org/
+ * Copyright   :  Written by and Copyright (C) 2001-2014 the
+ *                Privoxy team. https://www.privoxy.org/
  *
  *                Based on the Internet Junkbuster originally written
  *                by and Copyright (C) 1997 Anonymous Coders and
@@ -47,7 +46,7 @@ const char loaders_rcs[] = "$Id: loaders.c,v 1.74 2009/09/26 13:29:57 fabiankeil
 #include <ctype.h>
 #include <assert.h>
 
-#if !defined(_WIN32) && !defined(__OS2__)
+#if !defined(_WIN32)
 #include <unistd.h>
 #endif
 
@@ -63,8 +62,6 @@ const char loaders_rcs[] = "$Id: loaders.c,v 1.74 2009/09/26 13:29:57 fabiankeil
 #include "urlmatch.h"
 #include "encode.h"
 
-const char loaders_h_rcs[] = LOADERS_H_VERSION;
-
 /*
  * Currently active files.
  * These are also entered in the main linked list of files.
@@ -74,18 +71,57 @@ const char loaders_h_rcs[] = LOADERS_H_VERSION;
 static struct file_list *current_trustfile      = NULL;
 #endif /* def FEATURE_TRUST */
 
+#ifndef FUZZ
 static int load_one_re_filterfile(struct client_state *csp, int fileid);
+#endif
 
 static struct file_list *current_re_filterfile[MAX_AF_FILES]  = {
    NULL, NULL, NULL, NULL, NULL,
    NULL, NULL, NULL, NULL, NULL
 };
 
-/*
- * Pseudo filter type for load_one_re_filterfile
- */
-#define NO_NEW_FILTER -1
+/*********************************************************************
+ *
+ * Function    :  free_csp_resources
+ *
+ * Description :  Frees memory referenced by the csp that isn't
+ *                shared with other csps.
+ *
+ * Parameters  :
+ *          1  :  csp = Current client state (buffers, headers, etc...)
+ *
+ * Returns     :  N/A
+ *
+ *********************************************************************/
+void free_csp_resources(struct client_state *csp)
+{
+   freez(csp->ip_addr_str);
+#ifdef FEATURE_CLIENT_TAGS
+   freez(csp->client_address);
+#endif
+   freez(csp->listen_addr_str);
+   freez(csp->client_iob->buf);
+   freez(csp->iob->buf);
+   freez(csp->error_message);
 
+   if (csp->action->flags & ACTION_FORWARD_OVERRIDE &&
+      NULL != csp->fwd)
+   {
+      unload_forward_spec(csp->fwd);
+   }
+   free_http_request(csp->http);
+
+   destroy_list(csp->headers);
+#ifdef FEATURE_HTTPS_INSPECTION
+   destroy_list(csp->https_headers);
+#endif
+   destroy_list(csp->tags);
+#ifdef FEATURE_CLIENT_TAGS
+   destroy_list(csp->client_tags);
+#endif
+
+   free_current_action(csp->action);
+}
 
 /*********************************************************************
  *
@@ -114,21 +150,23 @@ static struct file_list *current_re_filterfile[MAX_AF_FILES]  = {
 unsigned int sweep(void)
 {
    struct file_list *fl, *nfl;
-   struct client_state *csp, *last_active;
+   struct client_state *csp;
+   struct client_states *last_active, *client_list;
    int i;
    unsigned int active_threads = 0;
 
    /* clear all of the file's active flags */
-   for ( fl = files->next; NULL != fl; fl = fl->next )
+   for (fl = files->next; NULL != fl; fl = fl->next)
    {
       fl->active = 0;
    }
 
    last_active = clients;
-   csp = clients->next;
+   client_list = clients->next;
 
-   while (NULL != csp)
+   while (NULL != client_list)
    {
+      csp = &client_list->csp;
       if (csp->flags & CSP_FLAG_ACTIVE)
       {
          /* Mark this client's files as active */
@@ -140,12 +178,12 @@ unsigned int sweep(void)
           */
          csp->config->config_file_list->active = 1;
 
-         /* 
+         /*
           * Actions files
           */
          for (i = 0; i < MAX_AF_FILES; i++)
          {
-            if (csp->actions_list[i])     
+            if (csp->actions_list[i])
             {
                csp->actions_list[i]->active = 1;
             }
@@ -156,7 +194,7 @@ unsigned int sweep(void)
           */
          for (i = 0; i < MAX_AF_FILES; i++)
          {
-            if (csp->rlist[i])     
+            if (csp->rlist[i])
             {
                csp->rlist[i]->active = 1;
             }
@@ -174,31 +212,15 @@ unsigned int sweep(void)
 
          active_threads++;
 
-         last_active = csp;
-         csp = csp->next;
+         last_active = client_list;
+         client_list = client_list->next;
       }
-      else 
+      else
       /*
        * This client is not active. Free its resources.
        */
       {
-         last_active->next = csp->next;
-
-         freez(csp->ip_addr_str);
-         freez(csp->iob->buf);
-         freez(csp->error_message);
-
-         if (csp->action->flags & ACTION_FORWARD_OVERRIDE &&
-             NULL != csp->fwd)
-         {
-            unload_forward_spec(csp->fwd);
-         }
-         free_http_request(csp->http);
-
-         destroy_list(csp->headers);
-         destroy_list(csp->tags);
-
-         free_current_action(csp->action);
+         last_active->next = client_list->next;
 
 #ifdef FEATURE_STATISTICS
          urls_read++;
@@ -208,9 +230,9 @@ unsigned int sweep(void)
          }
 #endif /* def FEATURE_STATISTICS */
 
-         freez(csp);
-         
-         csp = last_active->next;
+         freez(client_list);
+
+         client_list = last_active->next;
       }
    }
 
@@ -219,7 +241,7 @@ unsigned int sweep(void)
 
    while (fl != NULL)
    {
-      if ( ( 0 == fl->active ) && ( NULL != fl->unloader ) )
+      if ((0 == fl->active) && (NULL != fl->unloader))
       {
          nfl->next = fl->next;
 
@@ -290,23 +312,10 @@ int check_file_changed(const struct file_list * current,
       return 0;
    }
 
-   fs = (struct file_list *)zalloc(sizeof(struct file_list));
-   if (fs == NULL)
-   {
-      /* Out of memory error */
-      return 1;
-   }
-
-
-   fs->filename = strdup(filename);
+   fs = zalloc_or_die(sizeof(struct file_list));
+   fs->filename = strdup_or_die(filename);
    fs->lastmodified = statbuf->st_mtime;
 
-   if (fs->filename == NULL)
-   {
-      /* Out of memory error */
-      freez (fs);
-      return 1;
-   }
    *newfl = fs;
    return 1;
 }
@@ -357,7 +366,7 @@ jb_err simple_read_line(FILE *fp, char **dest, int *newline)
    p = buf;
 
 /*
- * Character codes.  If you have a wierd compiler and the following are
+ * Character codes.  If you have a weird compiler and the following are
  * incorrect, you also need to fix NEWLINE() in loaders.h
  */
 #define CHAR_CR '\r' /* ASCII 13 */
@@ -366,6 +375,7 @@ jb_err simple_read_line(FILE *fp, char **dest, int *newline)
    for (;;)
    {
       ch = getc(fp);
+
       if (ch == EOF)
       {
          if (len > 0)
@@ -422,6 +432,7 @@ jb_err simple_read_line(FILE *fp, char **dest, int *newline)
       }
       else if (ch == 0)
       {
+         /* XXX: Why do we allow this anyway? */
          *p = '\0';
          *dest = buf;
          return JB_ERR_OK;
@@ -539,36 +550,21 @@ jb_err edit_read_line(FILE *fp,
 
    if (raw_out)
    {
-      raw = strdup("");
-      if (NULL == raw)
-      {
-         return JB_ERR_MEMORY;
-      }
+      raw = strdup_or_die("");
    }
    if (prefix_out)
    {
-      prefix = strdup("");
-      if (NULL == prefix)
-      {
-         freez(raw);
-         return JB_ERR_MEMORY;
-      }
+      prefix = strdup_or_die("");
    }
    if (data_out)
    {
-      data = strdup("");
-      if (NULL == data)
-      {
-         freez(raw);
-         freez(prefix);
-         return JB_ERR_MEMORY;
-      }
+      data = strdup_or_die("");
    }
 
    /* Main loop.  Loop while we need more data & it's not EOF. */
 
-   while ( (contflag || is_empty)
-        && (JB_ERR_OK == (rval = simple_read_line(fp, &linebuf, newline))))
+   while ((contflag || is_empty)
+       && (JB_ERR_OK == (rval = simple_read_line(fp, &linebuf, newline))))
    {
       if (line_number)
       {
@@ -654,15 +650,12 @@ jb_err edit_read_line(FILE *fp,
       if (*linestart)
       {
          is_empty = 0;
-         if (data)
+         if (string_append(&data, linestart))
          {
-            if (string_append(&data, linestart))
-            {
-               freez(raw);
-               freez(prefix);
-               free(linebuf);
-               return JB_ERR_MEMORY;
-            }
+            freez(raw);
+            freez(prefix);
+            free(linebuf);
+            return JB_ERR_MEMORY;
          }
       }
 
@@ -735,41 +728,27 @@ jb_err edit_read_line(FILE *fp,
  *                and respects escaping of newline and comment char.
  *
  * Parameters  :
- *          1  :  buf = Buffer to use.
- *          2  :  buflen = Size of buffer in bytes.
- *          3  :  fp = File to read from
- *          4  :  linenum = linenumber in file
+ *          1  :  fp = File to read from
+ *          2  :  linenum = linenumber in file
+ *          3  :  buf = Pointer to a pointer to set to the data buffer.
  *
  * Returns     :  NULL on EOF or error
  *                Otherwise, returns buf.
  *
  *********************************************************************/
-char *read_config_line(char *buf, size_t buflen, FILE *fp, unsigned long *linenum)
+char *read_config_line(FILE *fp, unsigned long *linenum, char **buf)
 {
    jb_err err;
-   char *buf2 = NULL;
-   err = edit_read_line(fp, NULL, NULL, &buf2, NULL, linenum);
+   err = edit_read_line(fp, NULL, NULL, buf, NULL, linenum);
    if (err)
    {
       if (err == JB_ERR_MEMORY)
       {
          log_error(LOG_LEVEL_FATAL, "Out of memory loading a config file");
       }
-      return NULL;
-   }
-   else
-   {
-      assert(buf2);
-      if (strlen(buf2) + 1U > buflen)
-      {
-         log_error(LOG_LEVEL_FATAL,
-            "Max line limit reached. Linenumber: %u. Lenght: %u. Max lenght: %u.",
-            *linenum, strlen(buf2), buflen-1);
-      }
-      strlcpy(buf, buf2, buflen);
-      free(buf2);
-      return buf;
+      *buf = NULL;
    }
+   return *buf;
 }
 
 
@@ -795,7 +774,7 @@ static void unload_trustfile(void *f)
    {
       next = cur->next;
 
-      free_url_spec(cur->url);
+      free_pattern_spec(cur->url);
       free(cur);
 
       cur = next;
@@ -845,9 +824,9 @@ int load_trustfile(struct client_state *csp)
    FILE *fp;
 
    struct block_spec *b, *bl;
-   struct url_spec **tl;
+   struct pattern_spec **tl;
 
-   char  buf[BUFFER_SIZE], *p, *q;
+   char *buf = NULL;
    int reject, trusted;
    struct file_list *fs;
    unsigned long linenum = 0;
@@ -856,10 +835,7 @@ int load_trustfile(struct client_state *csp)
    if (!check_file_changed(current_trustfile, csp->config->trustfile, &fs))
    {
       /* No need to load */
-      if (csp)
-      {
-         csp->tlist = current_trustfile;
-      }
+      csp->tlist = current_trustfile;
       return(0);
    }
    if (!fs)
@@ -867,20 +843,17 @@ int load_trustfile(struct client_state *csp)
       goto load_trustfile_error;
    }
 
-   fs->f = bl = (struct block_spec *)zalloc(sizeof(*bl));
-   if (bl == NULL)
-   {
-      goto load_trustfile_error;
-   }
+   fs->f = bl = zalloc_or_die(sizeof(*bl));
 
    if ((fp = fopen(csp->config->trustfile, "r")) == NULL)
    {
       goto load_trustfile_error;
    }
+   log_error(LOG_LEVEL_INFO, "Loading trust file: %s", csp->config->trustfile);
 
    tl = csp->config->trust_list;
 
-   while (read_config_line(buf, sizeof(buf), fp, &linenum) != NULL)
+   while (read_config_line(fp, &linenum, &buf) != NULL)
    {
       trusted = 0;
       reject  = 1;
@@ -893,6 +866,9 @@ int load_trustfile(struct client_state *csp)
 
       if (*buf == '~')
       {
+         char *p;
+         char *q;
+
          reject = 0;
          p = buf;
          q = p+1;
@@ -905,15 +881,12 @@ int load_trustfile(struct client_state *csp)
       /* skip blank lines */
       if (*buf == '\0')
       {
+         freez(buf);
          continue;
       }
 
       /* allocate a new node */
-      if ((b = zalloc(sizeof(*b))) == NULL)
-      {
-         fclose(fp);
-         goto load_trustfile_error;
-      }
+      b = zalloc_or_die(sizeof(*b));
 
       /* add it to the list */
       b->next  = bl->next;
@@ -922,7 +895,7 @@ int load_trustfile(struct client_state *csp)
       b->reject = reject;
 
       /* Save the URL pattern */
-      if (create_url_spec(b->url, buf))
+      if (create_pattern_spec(b->url, buf))
       {
          fclose(fp);
          goto load_trustfile_error;
@@ -933,14 +906,15 @@ int load_trustfile(struct client_state *csp)
        */
       if (trusted)
       {
-         if(++trusted_referrers < MAX_TRUSTED_REFERRERS)
+         if (++trusted_referrers < MAX_TRUSTED_REFERRERS)
          {
             *tl++ = b->url;
          }
       }
+      freez(buf);
    }
 
-   if(trusted_referrers >= MAX_TRUSTED_REFERRERS) 
+   if (trusted_referrers >= MAX_TRUSTED_REFERRERS)
    {
       /*
        * FIXME: ... after Privoxy 3.0.4 is out.
@@ -964,17 +938,14 @@ int load_trustfile(struct client_state *csp)
    fs->next    = files->next;
    files->next = fs;
    current_trustfile = fs;
-
-   if (csp)
-   {
-      csp->tlist = fs;
-   }
+   csp->tlist = fs;
 
    return(0);
 
 load_trustfile_error:
    log_error(LOG_LEVEL_FATAL, "can't load trustfile '%s': %E",
-             csp->config->trustfile);
+      csp->config->trustfile);
+   freez(buf);
    return(-1);
 
 }
@@ -1018,7 +989,7 @@ static void unload_re_filterfile(void *f)
  *
  * Function    :  unload_forward_spec
  *
- * Description :  Unload the forward spec settings by freeing all 
+ * Description :  Unload the forward spec settings by freeing all
  *                memory referenced by members and the memory for
  *                the spec itself.
  *
@@ -1030,9 +1001,11 @@ static void unload_re_filterfile(void *f)
  *********************************************************************/
 void unload_forward_spec(struct forward_spec *fwd)
 {
-   free_url_spec(fwd->url);
+   free_pattern_spec(fwd->url);
    freez(fwd->gateway_host);
    freez(fwd->forward_host);
+   freez(fwd->auth_username);
+   freez(fwd->auth_password);
    free(fwd);
 
    return;
@@ -1072,7 +1045,7 @@ void unload_current_re_filterfile(void)
  *
  * Function    :  load_re_filterfiles
  *
- * Description :  Loads all the filterfiles. 
+ * Description :  Loads all the filterfiles.
  *                Generate a chained list of re_filterfile_spec's from
  *                the "FILTER: " blocks, compiling all their substitutions
  *                into chained lists of pcrs_job structs.
@@ -1113,7 +1086,7 @@ int load_re_filterfiles(struct client_state *csp)
  *
  * Function    :  load_one_re_filterfile
  *
- * Description :  Load a re_filterfile. 
+ * Description :  Load a re_filterfile.
  *                Generate a chained list of re_filterfile_spec's from
  *                the "FILTER: " blocks, compiling all their substitutions
  *                into chained lists of pcrs_job structs.
@@ -1131,8 +1104,7 @@ int load_one_re_filterfile(struct client_state *csp, int fileid)
    struct re_filterfile_spec *new_bl, *bl = NULL;
    struct file_list *fs;
 
-   char  buf[BUFFER_SIZE];
-   int error;
+   char *buf = NULL;
    unsigned long linenum = 0;
    pcrs_job *dummy, *lastjob = NULL;
 
@@ -1141,10 +1113,7 @@ int load_one_re_filterfile(struct client_state *csp, int fileid)
     */
    if (!check_file_changed(current_re_filterfile[fileid], csp->config->re_filterfile[fileid], &fs))
    {
-      if (csp)
-      {
-         csp->rlist[fileid] = current_re_filterfile[fileid];
-      }
+      csp->rlist[fileid] = current_re_filterfile[fileid];
       return(0);
    }
    if (!fs)
@@ -1152,7 +1121,7 @@ int load_one_re_filterfile(struct client_state *csp, int fileid)
       goto load_re_filterfile_error;
    }
 
-   /* 
+   /*
     * Open the file or fail
     */
    if ((fp = fopen(csp->config->re_filterfile[fileid], "r")) == NULL)
@@ -1160,12 +1129,14 @@ int load_one_re_filterfile(struct client_state *csp, int fileid)
       goto load_re_filterfile_error;
    }
 
-   /* 
+   log_error(LOG_LEVEL_INFO, "Loading filter file: %s", csp->config->re_filterfile[fileid]);
+
+   /*
     * Read line by line
     */
-   while (read_config_line(buf, sizeof(buf), fp, &linenum) != NULL)
+   while (read_config_line(fp, &linenum, &buf) != NULL)
    {
-      int new_filter = NO_NEW_FILTER;
+      enum filter_type new_filter = FT_INVALID_FILTER;
 
       if (strncmp(buf, "FILTER:", 7) == 0)
       {
@@ -1187,22 +1158,30 @@ int load_one_re_filterfile(struct client_state *csp, int fileid)
       {
          new_filter = FT_SERVER_HEADER_TAGGER;
       }
+#ifdef FEATURE_EXTERNAL_FILTERS
+      else if (strncmp(buf, "EXTERNAL-FILTER:", 16) == 0)
+      {
+         new_filter = FT_EXTERNAL_CONTENT_FILTER;
+      }
+#endif
 
       /*
        * If this is the head of a new filter block, make it a
        * re_filterfile spec of its own and chain it to the list:
        */
-      if (new_filter != NO_NEW_FILTER)
+      if (new_filter != FT_INVALID_FILTER)
       {
-         new_bl = (struct re_filterfile_spec  *)zalloc(sizeof(*bl));
-         if (new_bl == NULL)
-         {
-            goto load_re_filterfile_error;
-         }
+         new_bl = zalloc_or_die(sizeof(*bl));
          if (new_filter == FT_CONTENT_FILTER)
          {
             new_bl->name = chomp(buf + 7);
          }
+#ifdef FEATURE_EXTERNAL_FILTERS
+         else if (new_filter == FT_EXTERNAL_CONTENT_FILTER)
+         {
+            new_bl->name = chomp(buf + 16);
+         }
+#endif
          else
          {
             new_bl->name = chomp(buf + 21);
@@ -1219,19 +1198,20 @@ int load_one_re_filterfile(struct client_state *csp, int fileid)
             new_bl->description = html_encode(chomp(new_bl->description));
             if (NULL == new_bl->description)
             {
-               new_bl->description = strdup("Out of memory while encoding this filter's description to HTML");
+               new_bl->description = strdup_or_die("Out of memory while "
+                  "encoding filter description to HTML");
             }
          }
          else
          {
-            new_bl->description = strdup("No description available for this filter");
+            new_bl->description = strdup_or_die("No description available");
          }
 
-         new_bl->name = strdup(chomp(new_bl->name));
-         
+         new_bl->name = strdup_or_die(chomp(new_bl->name));
+
          /*
           * If this is the first filter block, chain it
-          * to the file_list rather than its (nonexistant)
+          * to the file_list rather than its (nonexistent)
           * predecessor
           */
          if (fs->f == NULL)
@@ -1246,23 +1226,50 @@ int load_one_re_filterfile(struct client_state *csp, int fileid)
          bl = new_bl;
 
          log_error(LOG_LEVEL_RE_FILTER, "Reading in filter \"%s\" (\"%s\")", bl->name, bl->description);
-
+#ifdef FEATURE_EXTENDED_STATISTICS
+         register_filter_for_statistics(bl->name);
+#endif
+         freez(buf);
          continue;
       }
 
-      /* 
-       * Else, save the expression, make it a pcrs_job
-       * and chain it into the current filter's joblist 
-       */
+#ifdef FEATURE_EXTERNAL_FILTERS
+      if ((bl != NULL) && (bl->type == FT_EXTERNAL_CONTENT_FILTER))
+      {
+         jb_err jb_error;
+         /* Save the code as "pattern", but do not compile anything. */
+         if (bl->patterns->first != NULL)
+         {
+            log_error(LOG_LEVEL_FATAL, "External filter '%s' contains several jobs. "
+               "Did you forget to escape a line break?",
+               bl->name);
+         }
+         jb_error = enlist(bl->patterns, buf);
+         if (JB_ERR_MEMORY == jb_error)
+         {
+            log_error(LOG_LEVEL_FATAL,
+               "Out of memory while enlisting external filter code \'%s\' for filter %s.",
+               buf, bl->name);
+         }
+         freez(buf);
+         continue;
+      }
+#endif
       if (bl != NULL)
       {
-         error = enlist(bl->patterns, buf);
-         if (JB_ERR_MEMORY == error)
+         int pcrs_error;
+         jb_err jb_error;
+         /*
+          * Save the expression, make it a pcrs_job
+          * and chain it into the current filter's joblist
+          */
+         jb_error = enlist(bl->patterns, buf);
+         if (JB_ERR_MEMORY == jb_error)
          {
             log_error(LOG_LEVEL_FATAL,
                "Out of memory while enlisting re_filter job \'%s\' for filter %s.", buf, bl->name);
          }
-         assert(JB_ERR_OK == error);
+         assert(JB_ERR_OK == jb_error);
 
          if (pcrs_job_is_dynamic(buf))
          {
@@ -1278,7 +1285,8 @@ int load_one_re_filterfile(struct client_state *csp, int fileid)
             bl->dynamic = 1;
             log_error(LOG_LEVEL_RE_FILTER,
                "Adding dynamic re_filter job \'%s\' to filter %s succeeded.", buf, bl->name);
-            continue;             
+            freez(buf);
+            continue;
          }
          else if (bl->dynamic)
          {
@@ -1289,13 +1297,16 @@ int load_one_re_filterfile(struct client_state *csp, int fileid)
              */
             log_error(LOG_LEVEL_RE_FILTER,
                "Adding static re_filter job \'%s\' to dynamic filter %s succeeded.", buf, bl->name);
+            freez(buf);
             continue;
          }
 
-         if ((dummy = pcrs_compile_command(buf, &error)) == NULL)
+         if ((dummy = pcrs_compile_command(buf, &pcrs_error)) == NULL)
          {
             log_error(LOG_LEVEL_ERROR,
-               "Adding re_filter job \'%s\' to filter %s failed with error %d.", buf, bl->name, error);
+               "Adding re_filter job \'%s\' to filter %s failed: %s",
+               buf, bl->name, pcrs_strerror(pcrs_error));
+            freez(buf);
             continue;
          }
          else
@@ -1314,17 +1325,19 @@ int load_one_re_filterfile(struct client_state *csp, int fileid)
       }
       else
       {
-         log_error(LOG_LEVEL_ERROR, "Ignoring job %s outside filter block in %s, line %d",
+         log_error(LOG_LEVEL_ERROR,
+            "Ignoring job %s outside filter block in %s, line %lu",
             buf, csp->config->re_filterfile[fileid], linenum);
       }
+      freez(buf);
    }
 
    fclose(fp);
 
-   /* 
+   /*
     * Schedule the now-obsolete old data for unloading
     */
-   if ( NULL != current_re_filterfile[fileid] )
+   if (NULL != current_re_filterfile[fileid])
    {
       current_re_filterfile[fileid]->unloader = unload_re_filterfile;
    }
@@ -1335,13 +1348,9 @@ int load_one_re_filterfile(struct client_state *csp, int fileid)
    fs->next    = files->next;
    files->next = fs;
    current_re_filterfile[fileid] = fs;
+   csp->rlist[fileid] = fs;
 
-   if (csp)
-   {
-      csp->rlist[fileid] = fs;
-   }
-
-   return( 0 );
+   return(0);
 
 load_re_filterfile_error:
    log_error(LOG_LEVEL_FATAL, "can't load re_filterfile '%s': %E",
@@ -1371,7 +1380,7 @@ void add_loader(int (*loader)(struct client_state *),
 {
    int i;
 
-   for (i=0; i < NLOADERS; i++)
+   for (i = 0; i < NLOADERS; i++)
    {
       if (config->loaders[i] == NULL)
       {
@@ -1405,7 +1414,7 @@ int run_loader(struct client_state *csp)
    int ret = 0;
    int i;
 
-   for (i=0; i < NLOADERS; i++)
+   for (i = 0; i < NLOADERS; i++)
    {
       if (csp->config->loaders[i] == NULL)
       {
@@ -1417,6 +1426,98 @@ int run_loader(struct client_state *csp)
 
 }
 
+/*********************************************************************
+ *
+ * Function    :  file_has_been_modified
+ *
+ * Description :  Helper function to check if a file has been changed
+ *
+ * Parameters  :
+ *          1  : filename = The name of the file to check
+ *          2  : last_known_modification = The time of the last known
+ *                                         modification
+ *
+ * Returns     :  TRUE if the file has been changed,
+ *                FALSE otherwise.
+ *
+ *********************************************************************/
+static int file_has_been_modified(const char *filename, time_t last_know_modification)
+{
+   struct stat statbuf[1];
+
+   if (stat(filename, statbuf) < 0)
+   {
+      /* Error, probably file not found which counts as change. */
+      return 1;
+   }
+
+   return (last_know_modification != statbuf->st_mtime);
+}
+
+
+/*********************************************************************
+ *
+ * Function    :  any_loaded_file_changed
+ *
+ * Description :  Helper function to check if any loaded file has been
+ *                changed since the time it has been loaded.
+ *
+ *                XXX: Should we cache the return value for x seconds?
+ *
+ * Parameters  :
+ *          1  : files_to_check = List of files to check
+ *
+ * Returns     : TRUE if any file has been changed,
+ *               FALSE otherwise.
+ *
+ *********************************************************************/
+int any_loaded_file_changed(const struct client_state *csp)
+{
+   const struct file_list *file_to_check = csp->config->config_file_list;
+   int i;
+
+   if (file_has_been_modified(file_to_check->filename, file_to_check->lastmodified))
+   {
+      return TRUE;
+   }
+
+   for (i = 0; i < MAX_AF_FILES; i++)
+   {
+      if (csp->actions_list[i])
+      {
+         file_to_check = csp->actions_list[i];
+         if (file_has_been_modified(file_to_check->filename, file_to_check->lastmodified))
+         {
+            return TRUE;
+         }
+      }
+   }
+
+   for (i = 0; i < MAX_AF_FILES; i++)
+   {
+      if (csp->rlist[i])
+      {
+         file_to_check = csp->rlist[i];
+         if (file_has_been_modified(file_to_check->filename, file_to_check->lastmodified))
+         {
+            return TRUE;
+         }
+      }
+   }
+
+#ifdef FEATURE_TRUST
+   if (csp->tlist)
+   {
+      if (file_has_been_modified(csp->tlist->filename, csp->tlist->lastmodified))
+      {
+         return TRUE;
+      }
+   }
+#endif /* def FEATURE_TRUST */
+
+   return FALSE;
+}
+
 
 /*
   Local Variables:
